unless Kernel.respond_to?(:require_relative)
	module Kernel
		def require_relative(path)
			require File.join(File.dirname(caller[0]), path.to_str)
		end
	end
end

require_relative 'helper'

class TestStruct < Struct.new(:apple,:bottle,:can,:donut); end
	
class TestStructformatter < Test::Unit::TestCase
	def test_simple_structures
		a = [1,'test',1.0,1e-9]
		assert_equal(
			"<array><element>1</element><element>test</element><element>1.0</element><element>1.0e-09</element></array>",
			a.to_xml
		)
		assert_equal(
			'[1,"test",1.0,1.0e-09]',
			a.to_json
		)
		# fails in 1.8
		assert_equal(
			'[1, "test", 1.0, 1.0e-09]',
			a.to_s
		)
		# fails in 1.8
		b = {'a'=>1, 'b'=>1.0, 'c'=>1e-9, 'd'=>'test'}
		assert_equal(
			"<hash><element><key>a</key><value>1</value></element><element><key>b</key><value>1.0</value></element><element><key>c</key><value>1.0e-09</value></element><element><key>d</key><value>test</value></element></hash>",
			b.to_xml
		)
		assert_equal(
			'{"a":1,"b":1.0,"c":1.0e-09,"d":"test"}',
			b.to_json
		)
		assert_equal(
			'{"a"=>1, "b"=>1.0, "c"=>1.0e-09, "d"=>"test"}',
			b.to_s
		)
		
		c = TestStruct.new(1,1.0,1e-9,'test')
		assert_equal(
			"<TestStruct apple='1' bottle='1.0' can='1.0e-09' donut='test' />",
			c.to_xml
		)
		assert_equal(
			'{"class":"TestStruct","apple":1,"bottle":1.0,"can":1.0e-09,"donut":"test"}',
			c.to_json
		)
		assert_equal(
			'1 1.0 1.0e-09 test',
			c.to_s
		)
	end
	
	def test_complex_structure
		a = TestStruct.new(
			[1,'test',{'a'=>1,'b'=>2.0},Date.parse("2010-10-10")],
			{'k'=>['j','<bad>']},
			TestStruct.new(5,4,3,2),
			true
		)
		# all of these will fail in 1.8
		assert_equal(
			"<TestStruct donut='true'><apples><apple>1</apple><apple>test</apple><apple><hash><element><key>a</key><value>1</value></element><element><key>b</key><value>2.0</value></element></hash></apple><apple>2010-10-10</apple></apples><bottles><HashElement><key>k</key><value><bottle><array><element>j</element><element>&lt;bad&gt;</element></array></bottle></value></HashElement></bottles><TestStruct apple='5' bottle='4' can='3' donut='2' /></TestStruct>",
			a.to_xml
		)
		assert_equal(
			'{"class":"TestStruct","apple":[1,"test",{"a":1,"b":2.0},"2010-10-10"],"bottle":{"k":["j","<bad>"]},"can":{"class":"TestStruct","apple":5,"bottle":4,"can":3,"donut":2},"donut":true}',
			a.to_json
		)
		assert_equal(
			'[1, "test", {"a"=>1, "b"=>2.0}, #<Date: 2010-10-10 ((2455480j,0s,0n),+0s,2299161j)>] {"k"=>["j", "<bad>"]} 5 4 3 2 true',
			a.to_s
		)
	end
	
	def test_csv_out
		a = [
			TestStruct.new(1,2,3,4),
			TestStruct.new(5,6,7,8),
			TestStruct.new(9,10,11,12),
			TestStruct.new(13,14,15,16)
		]
		assert_equal("apple,bottle,can,donut", a[0].to_s_header(","))
		assert_equal("1,2,3,4", a[0].to_s(","))
		assert_equal("5,6,7,8", a[1].to_s(","))
		assert_equal("9,10,11,12", a[2].to_s(","))
		assert_equal("13,14,15,16", a[3].to_s(","))
	end
end